@@ -1,6 +1,7 @@
 """This module contains helper classes and methods to assist with the
 collection of content to be posted to Mastodon."""
 # pylint: disable=E1136
+import asyncio
 import configparser
 import hashlib
 import logging
@@ -9,6 +10,7 @@ import re
 import sys
 from typing import Any
 from typing import Dict
+from typing import Iterable
 from typing import List
 from typing import Optional
 from typing import TypeVar
@@ -16,21 +18,26 @@ from urllib.error import URLError
 from urllib.parse import urlsplit
 from urllib.request import urlopen
 
+import asyncpraw.exceptions
+import asyncprawcore
 import httpx
-import praw
-import prawcore.exceptions
-from alive_progress import alive_bar
+from asyncpraw.models import Submission
 from bs4 import BeautifulSoup
 from gfycat.client import GfycatClient
 from gfycat.error import GfycatClientError
 from imgurpython import ImgurClient
 from imgurpython.helpers.error import ImgurClientError
 from PIL import Image as PILImage
-from praw.models import Submission
 from rich import print as rprint
+from tqdm.asyncio import tqdm
 
+from . import PROGRESS_BAR_FORMAT
+from . import USER_AGENT
 from .control import Configuration
 from .control import Secret
+from .control import SubredditConfig
+
+logger = logging.getLogger("Tootbot")
 
 FATAL_TOOTBOT_ERROR = "Tootbot cannot continue, now shutting down"
 
@@ -39,27 +46,36 @@ LMH = TypeVar("LMH", bound="LinkedMediaHelper")
 MA = TypeVar("MA", bound="MediaAttachment")
 
 
-# Function for downloading images from a URL to media folder
-def save_file(img_url: str, file_path: str, logger: logging.Logger) -> Optional[str]:
+async def get_file(img_url: str, file_path: str, progress_label: str) -> Optional[str]:
     """Utility method to save a file located at img_url to a file located at
     filepath.
 
     Arguments:
         img_url (string): url of imgur image to download
         file_path (string): directory and filename where to save the downloaded image to
-        logger (logger): logger to use for logging messages
+        progress_label (string): Message to add to progress bar
 
     Returns:
         file_path (string): path to downloaded image or None if no image was downloaded
     """
+    logger.debug("!!! 🎇🎇 Async Get File %s from %s", file_path, img_url)
     try:
-        with open(file_path, "wb") as image_file:
-            with httpx.stream("GET", img_url) as response:
-                response.raise_for_status()
-                for chunk in response.iter_bytes():
-                    image_file.write(chunk)
-
-            image_file.close()
+        client = httpx.AsyncClient()
+        meta = await client.head(url=img_url)
+        size = int(meta.headers["content-length"])
+        async with client.stream(method="GET", url=img_url) as response:
+            with open(file_path, "wb") as file_out:
+                async for chunk in tqdm(
+                    iterable=response.aiter_bytes(1),
+                    unit="iB",
+                    unit_scale=True,
+                    unit_divisor=1024,
+                    desc=f"{progress_label:.<60}",
+                    ncols=120,
+                    bar_format=PROGRESS_BAR_FORMAT,
+                    total=size,
+                ):
+                    file_out.write(chunk)
             return file_path
 
     except httpx.HTTPError as save_image_error:
@@ -67,19 +83,19 @@ def save_file(img_url: str, file_path: str, logger: logging.Logger) -> Optional[
         return None
 
 
-def get_secrets(logger: logging.Logger) -> Dict[str, Secret]:
+def get_secrets() -> Dict[str, Secret]:
     """Helper method to collect all api secrets either from already store
     secrets files or from user input."""
 
     secrets = {}
-    secrets["reddit"] = _get_reddit_secret(logger)
-    secrets["gfycat"] = _get_gfycat_secret(logger)
-    secrets["imgur"] = _get_imgur_secret(logger)
+    secrets["reddit"] = _get_reddit_secret()
+    secrets["gfycat"] = _get_gfycat_secret()
+    secrets["imgur"] = _get_imgur_secret()
 
     return secrets
 
 
-def _get_reddit_secret(logger: logging.Logger) -> Secret:
+def _get_reddit_secret() -> Secret:
     """Helper method to collect reddit secret from stored secrets file or from
     user input."""
     reddit_secrets_file = "reddit.secret"
@@ -96,7 +112,7 @@ def _get_reddit_secret(logger: logging.Logger) -> Secret:
         try:
             # create Reddit api connection and load posts from announcements subreddit
             # to confirm reddit connection works
-            reddit_client = praw.Reddit(
+            reddit_client = asyncpraw.Reddit(
                 user_agent="Tootbot",
                 client_id=reddit_agent,
                 client_secret=reddit_client_secret,
@@ -112,7 +128,7 @@ def _get_reddit_secret(logger: logging.Logger) -> Secret:
             }
             with open(reddit_secrets_file, "w", encoding="utf8") as new_reddit_secrets:
                 reddit_config.write(new_reddit_secrets)
-        except prawcore.exceptions.ResponseException as reddit_exception:
+        except asyncprawcore.AsyncPrawcoreException as reddit_exception:
             logger.error("Error while logging into Reddit: %s", reddit_exception)
             logger.error(FATAL_TOOTBOT_ERROR)
             sys.exit(1)
@@ -126,7 +142,7 @@ def _get_reddit_secret(logger: logging.Logger) -> Secret:
     )
 
 
-def _get_gfycat_secret(logger: logging.Logger) -> Secret:
+def _get_gfycat_secret() -> Secret:
     """_get_gfycat_secrets checks if the Gfycat api secrets file exists.
 
     - If the file exists, this method reads the files and returns the secrets
@@ -180,7 +196,7 @@ def _get_gfycat_secret(logger: logging.Logger) -> Secret:
     )
 
 
-def _get_imgur_secret(logger: logging.Logger) -> Secret:
+def _get_imgur_secret() -> Secret:
     """_get_imgur_secrets checks if the Imgur api secrets file exists.
 
     - If the file exists, this method reads the imgur secrets file and returns the
@@ -239,50 +255,60 @@ class RedditHelper:
 
     # Check if reddit access details in 'reddit.secret' file has already been set up
     # and load it, otherwise guide user through setting it up.
-    def __init__(
-        self: RH,
-        config: Configuration,
-        api_secret: Secret,
-        user_agent: str = "Tootbot",
-    ) -> None:
+    def __init__(self: RH, config: Configuration, api_secret: Secret) -> None:
         self.config = config
-        self.logger = config.bot.logger
-        self.posts: Dict[str, praw.models.Submission] = {}
+        self.posts: Dict[str, asyncpraw.models.Submission] = {}
+        self.api_secret = api_secret
 
-        self.reddit_connection = praw.Reddit(
-            user_agent=user_agent,
-            client_id=api_secret.client_id,
-            client_secret=api_secret.client_secret,
+    async def get_all_reddit_posts(self: RH) -> None:
+        """Collects posts from all configured subreddits."""
+        tasks = []
+
+        reddit_con = asyncpraw.Reddit(
+            user_agent=USER_AGENT,
+            client_id=self.api_secret.client_id,
+            client_secret=self.api_secret.client_secret,
         )
 
-    def get_all_reddit_posts(self: RH) -> None:
-        """Collects posts from all configured subreddits."""
-        title = "Collecting posts from reddit "
-        with alive_bar(
-            title=f"{title:.<60}",
-            enrich_print=False,
-            dual_line=True,
-        ) as progress_bar:
-            for subreddit in self.config.subreddits:
-                try:
-                    progress_bar.text = f"Getting posts from: r/{subreddit.name}"
-                    subreddit_info = self.reddit_connection.subreddit(subreddit.name)
-                    subreddit_posts: Dict[str, praw.models.Submission] = {}
-                    for submission in subreddit_info.hot(
-                        limit=self.config.reddit.post_limit
-                    ):
-                        subreddit_posts[submission.id] = submission
-                        progress_bar()  # pylint: disable=not-callable
-                    self.posts[subreddit.tags] = subreddit_posts
-                except (
-                    praw.exceptions.PRAWException,
-                    prawcore.exceptions.PrawcoreException,
-                ) as reddit_error:
-                    self.logger.warning(
-                        "Error when getting reddit posts from r/%s: %s",
-                        subreddit.name,
-                        reddit_error,
-                    )
+        for subreddit in self.config.subreddits:
+            tasks.append(self.get_reddit_posts(subreddit, reddit_con))
+        progress_title = "Processing Subreddits"
+        task_results = await tqdm.gather(
+            *tasks,
+            desc=f"{progress_title:.<60}",
+            ncols=120,
+            bar_format=PROGRESS_BAR_FORMAT,
+            total=len(tasks),
+        )
+
+        for result in task_results:
+            self.posts.update(result)
+
+        await reddit_con.close()
+
+    async def get_reddit_posts(
+        self: RH,
+        subreddit: SubredditConfig,
+        reddit_con: asyncpraw.Reddit,
+    ) -> Dict[str, asyncpraw.models.Submission]:
+        """Collects posts considered hot from configured sub/multi-reddits."""
+        posts = {}
+        logger.debug("Retrieving posts from %s", subreddit.name)
+        try:
+            subreddit_info = await reddit_con.subreddit(subreddit.name)
+            subreddit_posts: Dict[str, asyncpraw.models.Submission] = {}
+            async for submission in subreddit_info.hot(
+                limit=self.config.reddit.post_limit
+            ):
+                subreddit_posts[submission.id] = submission
+            posts[subreddit.tags] = subreddit_posts
+        except asyncprawcore.AsyncPrawcoreException as reddit_error:
+            logger.warning(
+                "Error when getting reddit posts from r/%s: %s",
+                subreddit.name,
+                reddit_error,
+            )
+        return posts
 
     def winnow_reddit_posts(self: RH) -> None:
         """Filters out reddit posts according to configuration and whether it
@@ -294,59 +320,49 @@ class RedditHelper:
         stickied_allowed = self.config.reddit.stickied_allowed
 
         title = "Winnowing chaff "
-        with alive_bar(
-            title=f"{title:.<60}",
-            enrich_print=False,
-        ) as progress_bar:
-            for posts in self.posts.values():
-                post_ids = list(posts.keys())
-                for post_id in post_ids:
+        for posts in tqdm(
+            self.posts.values(),
+            desc=f"{title:.<60}",
+            total=len(self.posts),
+            ncols=120,
+            bar_format=PROGRESS_BAR_FORMAT,
+        ):
+            post_ids = list(posts.keys())
+            for post_id in post_ids:
 
-                    if posts[post_id].over_18 and not nsfw_allowed:
-                        # Skip over NSFW posts if they are disabled in the config file
-                        self.logger.debug("Skipping %s, it is marked as NSFW", post_id)
-                        del posts[post_id]
-                        progress_bar()  # pylint: disable=not-callable
-                        continue
+                if posts[post_id].over_18 and not nsfw_allowed:
+                    # Skip over NSFW posts if they are disabled in the config file
+                    logger.debug("Skipping %s, it is marked as NSFW", post_id)
+                    del posts[post_id]
+                    continue
 
-                    if posts[post_id].is_self and not self_posts_allowed:
-                        # Skip over NSFW posts if they are disabled in the config file
-                        self.logger.debug("Skipping %s, it is a self post", post_id)
-                        del posts[post_id]
-                        progress_bar()  # pylint: disable=not-callable
-                        continue
+                if posts[post_id].is_self and not self_posts_allowed:
+                    # Skip over NSFW posts if they are disabled in the config file
+                    logger.debug("Skipping %s, it is a self post", post_id)
+                    del posts[post_id]
+                    continue
 
-                    if posts[post_id].spoiler and not spoilers_allowed:
-                        # Skip over posts marked as spoilers if they are disabled in
-                        # the config file
-                        self.logger.debug(
-                            "Skipping %s, it is marked as a spoiler", post_id
-                        )
-                        del posts[post_id]
-                        progress_bar()  # pylint: disable=not-callable
-                        continue
+                if posts[post_id].spoiler and not spoilers_allowed:
+                    # Skip over posts marked as spoilers if they are disabled in
+                    # the config file
+                    logger.debug("Skipping %s, it is marked as a spoiler", post_id)
+                    del posts[post_id]
+                    continue
 
-                    if posts[post_id].stickied and not stickied_allowed:
-                        self.logger.debug("Skipping %s, it is stickied", post_id)
-                        del posts[post_id]
-                        progress_bar()  # pylint: disable=not-callable
-                        continue
+                if posts[post_id].stickied and not stickied_allowed:
+                    logger.debug("Skipping %s, it is stickied", post_id)
+                    del posts[post_id]
+                    continue
 
-                    if recorder.duplicate_check(post_id):
-                        self.logger.debug(
-                            "Skipping %s, it has already been tooted", post_id
-                        )
-                        del posts[post_id]
-                        progress_bar()  # pylint: disable=not-callable
-                        continue
+                if recorder.duplicate_check(post_id):
+                    logger.debug("Skipping %s, it has already been tooted", post_id)
+                    del posts[post_id]
+                    continue
 
-                    if recorder.duplicate_check(posts[post_id].url):
-                        self.logger.debug(
-                            "Skipping %s, it has already been tooted", post_id
-                        )
-                        del posts[post_id]
-                        progress_bar()  # pylint: disable=not-callable
-                        continue
+                if recorder.duplicate_check(posts[post_id].url):
+                    logger.debug("Skipping %s, it has already been tooted", post_id)
+                    del posts[post_id]
+                    continue
 
     def get_caption(
         self: RH,
@@ -409,7 +425,6 @@ class LinkedMediaHelper:
         gfycat_secrets: Secret,
         imgur_secrets: Secret,
     ) -> None:
-        self.logger = config.bot.logger
         self.save_dir = config.media.folder
 
         try:
@@ -422,15 +437,17 @@ class LinkedMediaHelper:
                 client_secret=gfycat_secrets.client_secret,
             )
         except ImgurClientError as imgur_error:
-            self.logger.error("Error on creating ImgurClient: %s", imgur_error)
-            self.logger.error(FATAL_TOOTBOT_ERROR)
+            logger.error("Error on creating ImgurClient: %s", imgur_error)
+            logger.error(FATAL_TOOTBOT_ERROR)
             sys.exit(1)
         except GfycatClientError as gfycat_error:
-            self.logger.error("Error on creating GfycatClient: %s", gfycat_error)
-            self.logger.error(FATAL_TOOTBOT_ERROR)
+            logger.error("Error on creating GfycatClient: %s", gfycat_error)
+            logger.error(FATAL_TOOTBOT_ERROR)
             sys.exit(1)
 
-    def get_imgur_image(self: LMH, img_url: str, max_images: int = 4) -> List[str]:
+    async def get_imgur_image(
+        self: LMH, img_url: str, max_images: int = 4
+    ) -> List[str]:
         """get_imgur_image downloads images from imgur.
 
         Arguments:
@@ -447,9 +464,7 @@ class LinkedMediaHelper:
         regex_match = re.search(regex, img_url, flags=0)
 
         if not regex_match:
-            self.logger.error(
-                "Could not identify Imgur image/gallery ID at: %s", img_url
-            )
+            logger.error("Could not identify Imgur image/gallery ID at: %s", img_url)
             return []
 
         # Get the Imgur image/gallery ID
@@ -459,49 +474,43 @@ class LinkedMediaHelper:
 
         # Download and process individual images (up to max_images)
         imgur_paths: List[str] = []
-        title = "Downloading image(s) from Imgur"
-        with alive_bar(
-            title=f"{title:.<60}",
-            total=min(len(image_urls), 4),
-            enrich_print=False,
-            dual_line=True,
-        ) as progress_bar:
-            for image_url in image_urls:
-                progress_bar.text = f"processing {image_url}"
-                # If the URL is a GIFV or MP4 link, change it to the GIF version
-                file_extension = os.path.splitext(image_url)[-1].lower()
-                if file_extension == ".gifv":
-                    file_extension = ".gif"
-                    image_url = image_url.replace(".gifv", ".gif")
-                elif file_extension == ".mp4":
-                    file_extension = ".gif"
-                    image_url = image_url.replace(".mp4", ".gif")
+        for image_url in image_urls:
+            # If the URL is a GIFV or MP4 link, change it to the GIF version
+            file_extension = os.path.splitext(image_url)[-1].lower()
+            if file_extension == ".gifv":
+                file_extension = ".gif"
+                image_url = image_url.replace(".gifv", ".gif")
+            elif file_extension == ".mp4":
+                file_extension = ".gif"
+                image_url = image_url.replace(".mp4", ".gif")
 
-                # Download the image
-                file_path = (
-                    self.save_dir
-                    + "/"
-                    + imgur_id
-                    + "_"
-                    + str(len(imgur_paths))
-                    + file_extension
-                )
-                self.logger.debug(
-                    "Downloading Imgur image at URL %s to %s", image_url, file_path
-                )
-                progress_bar.text = f"{file_path} <-- {image_url}"
-                current_image = save_file(image_url, file_path, self.logger)
+            # Download the image
+            file_path = (
+                self.save_dir
+                + "/"
+                + imgur_id
+                + "_"
+                + str(len(imgur_paths))
+                + file_extension
+            )
+            logger.debug(
+                "Downloading Imgur image at URL %s to %s", image_url, file_path
+            )
+            saved_paths = await get_file(
+                img_url=image_url,
+                file_path=file_path,
+                progress_label="Downloading Imgur image",
+            )
 
-                # Imgur will sometimes return a single-frame thumbnail
-                # instead of a GIF, so we need to check for this
-                if current_image and (
-                    file_extension != ".gif" or self._check_imgur_gif(file_path)
-                ):
-                    imgur_paths.append(current_image)
-                    progress_bar()  # pylint: disable=not-callable
+            # Imgur will sometimes return a single-frame thumbnail
+            # instead of a GIF, so we need to check for this
+            if saved_paths and (
+                file_extension != ".gif" or self._check_imgur_gif(saved_paths)
+            ):
+                imgur_paths.append(saved_paths)
 
-                if len(imgur_paths) == max_images:
-                    break
+            if len(imgur_paths) == max_images:
+                break
 
         return imgur_paths
 
@@ -519,7 +528,7 @@ class LinkedMediaHelper:
         image_urls = []
         try:
             if any(s in img_url for s in ("/a/", "/gallery/")):  # Gallery links
-                self.logger.debug("Imgur link points to gallery: %s", img_url)
+                logger.debug("Imgur link points to gallery: %s", img_url)
                 images = self.imgur_client.get_album_images(imgur_id)
                 for image in images:
                     image_urls.append(image.link)
@@ -528,7 +537,7 @@ class LinkedMediaHelper:
                 image_urls = [imgur_img.link]  # pylint: disable=no-member
 
         except ImgurClientError as imgur_error:
-            self.logger.error("Could not get information from imgur: %s", imgur_error)
+            logger.error("Could not get information from imgur: %s", imgur_error)
         return image_urls
 
     def _check_imgur_gif(self: LMH, file_path: str) -> bool:
@@ -546,16 +555,16 @@ class LinkedMediaHelper:
         img.close()
 
         if mime != "image/gif":
-            self.logger.warning("Imgur: not a GIF, not posting")
+            logger.warning("Imgur: not a GIF, not posting")
             try:
                 os.remove(file_path)
             except OSError as remove_error:
-                self.logger.error("Error while deleting media file: %s", remove_error)
+                logger.error("Error while deleting media file: %s", remove_error)
             return False
 
         return True
 
-    def get_gfycat_image(self: LMH, img_url: str) -> Optional[str]:
+    async def get_gfycat_image(self: LMH, img_url: str) -> Optional[str]:
         """get_gfycat_image downloads full resolution images from gfycat.
 
         Arguments:
@@ -581,29 +590,24 @@ class LinkedMediaHelper:
             httpx.HTTPError,
             GfycatClientError,
         ) as gfycat_error:
-            self.logger.error("Error downloading Gfycat link: %s", gfycat_error)
+            logger.error("Error downloading Gfycat link: %s", gfycat_error)
             return None
 
         if gfycat_url == "":
-            self.logger.debug("Empty Gfycat URL; no attachment to download")
+            logger.debug("Empty Gfycat URL; no attachment to download")
             return None
 
-        self.logger.debug("Downloading Gfycat at URL %s to %s", gfycat_url, file_path)
+        logger.debug("Downloading Gfycat at URL %s to %s", gfycat_url, file_path)
 
-        title = "Downloading Gfycat image/video"
-        with alive_bar(
-            title=f"{title:.<60}",
-            total=1,
-            enrich_print=False,
-            dual_line=True,
-        ) as progress_bar:
-            progress_bar.text = f"{file_path} <-- {gfycat_url}"
-            saved_file = save_file(gfycat_url, file_path, self.logger)
-            progress_bar()  # pylint: disable=not-callable
+        saved_paths = await get_file(
+            img_url=gfycat_url,
+            file_path=file_path,
+            progress_label="Downloading Gfycat image/video",
+        )
 
-        return saved_file
+        return saved_paths
 
-    def get_reddit_image(self: LMH, img_url: str) -> Optional[str]:
+    async def get_reddit_image(self: LMH, img_url: str) -> Optional[str]:
         """get_reddit_image downloads full resolution images from i.reddit or
         reddituploads.com.
 
@@ -624,27 +628,22 @@ class LinkedMediaHelper:
             img_url += ".jpg"
         # Download the file
         file_path = self.save_dir + "/" + file_name
-        self.logger.debug(
+        logger.debug(
             "Downloading file at URL %s to %s, file type identified as %s",
             img_url,
             file_path,
             file_extension,
         )
 
-        title = "Downloading reddit image"
-        with alive_bar(
-            title=f"{title:.<60}",
-            total=1,
-            enrich_print=False,
-            dual_line=True,
-        ) as progress_bar:
-            progress_bar.text = f"{file_name} <-- {img_url}"
-            saved_file = save_file(img_url, file_path, self.logger)
-            progress_bar()  # pylint: disable=not-callable
+        saved_path = await get_file(
+            img_url=img_url,
+            file_path=file_path,
+            progress_label="Downloading reddit image",
+        )
 
-        return saved_file
+        return saved_path
 
-    def get_reddit_gallery(
+    async def get_reddit_gallery(
         self: LMH,
         reddit_post: Submission,
         max_images: int = 4,
@@ -661,39 +660,32 @@ class LinkedMediaHelper:
             file_paths (List[str]) a list of the paths to downloaded files. If no
             images have been downloaded, and empty list will be returned.
         """
-        file_paths = []
+        file_paths: List[Optional[str]] = []
         gallery_items: List[Dict[str, Any]] = reddit_post.gallery_data["items"]
-        title = "Downloading images from reddit gallery"
-        with alive_bar(
-            title=f"{title:.<60}",
-            total=min(len(gallery_items), 4),
-            enrich_print=False,
-            dual_line=True,
-        ) as progress_bar:
+        tasks: List[Any] = []
+        for item in gallery_items:
+            media_id = item["media_id"]
+            meta = reddit_post.media_metadata[media_id]
+            logger.debug("Media Metadata: %s", meta)
+            if "e" in meta and meta["e"] == "Image":
+                source = meta["s"]
+                save_path = (
+                    self.save_dir + "/" + media_id + "." + meta["m"].split("/")[1]
+                )
+                logger.debug(
+                    "Gallery file_path, source: %s - %s", save_path, source["u"]
+                )
+                tasks.append(get_file(source["u"], save_path, f"image {len(tasks)+1}"))
 
-            for item in gallery_items:
-                media_id = item["media_id"]
-                meta = reddit_post.media_metadata[media_id]
-                self.logger.debug("Media Metadata: %s", meta)
-                if "e" in meta and meta["e"] == "Image":
-                    source = meta["s"]
-                    save_path = (
-                        self.save_dir + "/" + media_id + "." + meta["m"].split("/")[1]
-                    )
-                    self.logger.debug(
-                        "Gallery file_path, source: %s - %s", save_path, source["u"]
-                    )
-                    progress_bar.text = f"{save_path} <-- {source['u']}"
-                    file_paths.append(save_file(source["u"], save_path, self.logger))
+                if len(tasks) == max_images:
+                    break
 
-                    progress_bar()  # pylint: disable=not-callable
-
-                    if len(file_paths) == max_images:
-                        break
+        rprint("Donwloading images from reddit gallery")
+        file_paths = await asyncio.gather(*tasks)
 
         return file_paths
 
-    def get_reddit_video(self: LMH, reddit_post: Submission) -> Optional[str]:
+    async def get_reddit_video(self: LMH, reddit_post: Submission) -> Optional[str]:
         """get_reddit_video downloads full resolution video from i.reddit or
         reddituploads.
 
@@ -706,24 +698,17 @@ class LinkedMediaHelper:
         """
         # Get URL for MP4 version of reddit video
         video_url = reddit_post.media["reddit_video"]["fallback_url"]
-        file_path = self.save_dir + "/" + reddit_post.id + ".mp4"
-        self.logger.debug(
-            "Downloading Reddit video at URL %s to %s", video_url, file_path
+        file_path: str = self.save_dir + "/" + reddit_post.id + ".mp4"
+        logger.debug("Downloading Reddit video at URL %s to %s", video_url, file_path)
+
+        saved_path = await get_file(
+            img_url=video_url,
+            file_path=file_path,
+            progress_label="Downloading video from Reddit",
         )
-        title = "Downloading video from Reddit"
-        with alive_bar(
-            title=f"{title:.<60}",
-            total=1,
-            enrich_print=False,
-            dual_line=True,
-        ) as progress_bar:
-            progress_bar.text = f"{file_path} <-- {video_url}"
-            saved_file = save_file(video_url, file_path, self.logger)
-            progress_bar()  # pylint: disable=not-callable
+        return saved_path
 
-        return saved_file
-
-    def get_giphy_image(self: LMH, img_url: str) -> Optional[str]:
+    async def get_giphy_image(self: LMH, img_url: str) -> Optional[str]:
         """get_giphy_image downloads full or low resolution image from giphy.
 
         Arguments:
@@ -740,7 +725,7 @@ class LinkedMediaHelper:
         )
         match = re.search(regex, img_url, flags=0)
         if not match:
-            self.logger.error("Could not identify Giphy ID in this URL: %s", img_url)
+            logger.error("Could not identify Giphy ID in this URL: %s", img_url)
             return None
 
         # Get the Giphy ID
@@ -749,22 +734,17 @@ class LinkedMediaHelper:
         giphy_url = "https://media.giphy.com/media/" + giphy_id + "/giphy.mp4"
         file_path = self.save_dir + "/" + giphy_id + "giphy.mp4"
 
-        title = ("Downloading Giphy image/video",)
-        with alive_bar(
-            title=f"{title:<60}",
-            total=1,
-            enrich_print=False,
-            dual_line=True,
-        ) as progress_bar:
-            progress_bar.text = f"{file_path} <-- {giphy_url}"
-            giphy_file = save_file(giphy_url, file_path, self.logger)
-            progress_bar()  # pylint: disable=not-callable
+        saved_paths = await get_file(
+            img_url=giphy_url,
+            file_path=file_path,
+            progress_label="Downloading Giphy image/video",
+        )
 
-        self.logger.debug("Downloaded Giphy at URL %s to %s", giphy_url, file_path)
+        logger.debug("Downloaded Giphy at URL %s to %s", giphy_url, saved_paths)
 
-        return giphy_file
+        return saved_paths
 
-    def get_generic_image(self: LMH, img_url: str) -> Optional[str]:
+    async def get_generic_image(self: LMH, img_url: str) -> Optional[str]:
         """get_generic_image downloads image or video from a generic url to a
         media file.
 
@@ -779,7 +759,7 @@ class LinkedMediaHelper:
         regex = r"^https?://"
         match = re.search(regex, img_url, flags=0)
         if not match:
-            self.logger.debug("Post link is not a full link: %s", img_url)
+            logger.debug("Post link is not a full link: %s", img_url)
             return None
 
         # Check if URL is an image or MP4 file, based on the MIME type
@@ -797,30 +777,25 @@ class LinkedMediaHelper:
                 meta = img_site.info()
 
         except (URLError, UnicodeEncodeError) as url_error:
-            self.logger.error("Error while opening URL %s", url_error)
+            logger.error("Error while opening URL %s", url_error)
             return None
 
         if meta["content-type"] not in image_formats:
-            self.logger.debug("URL does not point to a valid image file: %s", img_url)
+            logger.debug("URL does not point to a valid image file: %s", img_url)
             return None
 
         # URL appears to be an image, so download it
         file_name = os.path.basename(urlsplit(img_url).path)
         file_path = self.save_dir + "/" + file_name
-        self.logger.debug("Downloading file at URL %s to %s", img_url, file_path)
+        logger.debug("Downloading file at URL %s to %s", img_url, file_path)
 
-        title = "Downloading generic image"
-        with alive_bar(
-            title=f"{title:.<60}",
-            total=1,
-            enrich_print=False,
-            dual_line=True,
-        ) as progress_bar:
-            progress_bar.text = f"{file_path} <-- {img_url}"
-            saved_file = save_file(img_url, file_path, self.logger)
-            progress_bar()  # pylint: disable=not-callable
+        saved_path = await get_file(
+            img_url=img_url,
+            file_path=file_path,
+            progress_label="Downloading generic image",
+        )
 
-        return saved_file
+        return saved_path
 
 
 class MediaAttachment:
@@ -828,22 +803,21 @@ class MediaAttachment:
     videos to include in a reddit post to be shared on Mastodon."""
 
     def __init__(
-        self: MA,
-        reddit_post: Submission,
-        image_helper: LinkedMediaHelper,
-        logger: logging.Logger,
+        self: MA, reddit_post: Submission, image_helper: LinkedMediaHelper
     ) -> None:
 
-        self.media_paths = {}
+        self.media_paths: Dict[str, str] = {}
         self.reddit_post = reddit_post
         self.media_url = self.reddit_post.url
         self.image_helper = image_helper
-        self.logger = logger
 
-        logger.debug("URL for post(%s): %s ", reddit_post, self.media_url)
+    async def get_media_files(self: MA) -> None:
+        """Downloads media files linked to given reddit_post."""
+        logger.debug("URL for post(%s): %s ", self.reddit_post, self.media_url)
 
-        for media_path in self.get_media():
-            self.logger.debug("Media path for checksum calculation: %s", media_path)
+        media_paths = await self.get_media()
+        for media_path in media_paths:
+            logger.debug("Media path for checksum calculation: %s", media_path)
             if media_path:
                 sha256 = hashlib.sha256()
                 with open(media_path, "rb") as media_file:
@@ -863,9 +837,9 @@ class MediaAttachment:
             for media_path in self.media_paths.values():
                 if media_path is not None:
                     os.remove(media_path)
-                    self.logger.debug("Deleted media file at %s", media_path)
+                    logger.debug("Deleted media file at %s", media_path)
         except OSError as delete_error:
-            self.logger.error("Error while deleting media file: %s", delete_error)
+            logger.error("Error while deleting media file: %s", delete_error)
 
         self.media_paths = {}
         self.media_url = None
@@ -880,51 +854,61 @@ class MediaAttachment:
             media_path = self.media_paths[checksum]
             if media_path is not None:
                 os.remove(media_path)
-                self.logger.debug("Deleted media file at %s", media_path)
+                logger.debug("Deleted media file at %s", media_path)
             self.media_paths.pop(checksum)
         except OSError as delete_error:
-            self.logger.error("Error while deleting media file: %s", delete_error)
+            logger.error("Error while deleting media file: %s", delete_error)
 
-    def get_media(self: MA) -> List[Optional[str]]:
+    async def get_media(self: MA) -> List[Optional[str]]:
         """Determines which method to call depending on which site the
         media_url is pointing to."""
         if not os.path.exists(self.image_helper.save_dir):
             os.makedirs(self.image_helper.save_dir)
-            self.logger.debug(
+            logger.debug(
                 "Media folder not found, created new folder: %s",
                 self.image_helper.save_dir,
             )
 
         file_paths = []
+        saved_media_path: Optional[str]
+        saved_media_paths: Iterable[Optional[str]]
 
         # Download and save the linked image
         if any(s in self.media_url for s in ("i.redd.it", "i.reddituploads.com")):
-            file_paths.append(self.image_helper.get_reddit_image(self.media_url))
+            saved_media_path = await self.image_helper.get_reddit_image(self.media_url)
+            file_paths.append(saved_media_path)
         elif "v.redd.it" in self.media_url and not self.reddit_post.media:
-            self.logger.error(
+            logger.error(
                 "Reddit API returned no media for this URL: %s", self.media_url
             )
         elif "v.redd.it" in self.media_url:
-            file_paths.append(self.image_helper.get_reddit_video(self.reddit_post))
+            saved_media_path = await self.image_helper.get_reddit_video(
+                self.reddit_post
+            )
+            file_paths.append(saved_media_path)
 
         elif "imgur.com" in self.media_url:
-            self.logger.debug("Reddit post %s links to Imgur", self.reddit_post.id)
-            file_paths.extend(self.image_helper.get_imgur_image(self.media_url))
+            saved_media_paths = await self.image_helper.get_imgur_image(self.media_url)
+            file_paths.extend(saved_media_paths)
 
         elif "gfycat.com" in self.media_url:
-            file_paths.append(self.image_helper.get_gfycat_image(self.media_url))
+            saved_media_path = await self.image_helper.get_gfycat_image(self.media_url)
+            file_paths.append(saved_media_path)
 
         elif "giphy.com" in self.media_url:
-            file_paths.append(self.image_helper.get_giphy_image(self.media_url))
+            saved_media_path = await self.image_helper.get_giphy_image(self.media_url)
+            file_paths.append(saved_media_path)
 
         elif "reddit.com/gallery/" in self.media_url:  # Need to check for gallery post
             if hasattr(self.reddit_post, "is_gallery"):
-                self.logger.debug("%s is a gallery post", self.reddit_post.id)
-                file_paths.extend(
-                    self.image_helper.get_reddit_gallery(self.reddit_post)
+                logger.debug("%s is a gallery post", self.reddit_post.id)
+                save_media_paths = await self.image_helper.get_reddit_gallery(
+                    self.reddit_post
                 )
+                file_paths.extend(save_media_paths)
 
         else:
-            file_paths.append(self.image_helper.get_generic_image(self.media_url))
+            saved_media_path = await self.image_helper.get_generic_image(self.media_url)
+            file_paths.append(saved_media_path)
 
         return file_paths
