@@ -31,7 +31,7 @@ class PostRecorder:
         # Make sure DB tables exist
         db_cursor = self.db_connection.cursor()
         db_cursor.execute(
-            "CREATE TABLE IF NOT EXISTS post " "(id TEXT PRIMARY KEY) " "WITHOUT ROWID"
+            "CREATE TABLE IF NOT EXISTS post (id TEXT PRIMARY KEY) WITHOUT ROWID"
         )
         db_cursor.execute(
             "CREATE TABLE IF NOT EXISTS share "
