@@ -11,16 +11,19 @@ from typing import List
 from typing import TypeVar
 
 import magic
-from alive_progress import alive_bar
 from mastodon import Mastodon
 from mastodon import MastodonError
 from rich import print as rprint
+from tqdm.asyncio import tqdm
 
+from . import PROGRESS_BAR_FORMAT
 from .collect import LinkedMediaHelper
 from .collect import MediaAttachment
 from .collect import RedditHelper
 from .control import Configuration
 
+logger = logging.getLogger("Tootbot")
+
 MP = TypeVar("MP", bound="MastodonPublisher")
 
 
@@ -52,20 +55,18 @@ class MastodonPublisher:
             # Make sure authentication is working
             userinfo = self.mastodon.account_verify_credentials()
             mastodon_username = userinfo["username"]
-            config.bot.logger.debug(
+            logger.debug(
                 "Successfully authenticated on %s as @%s",
                 self.mastodon_config.domain,
                 mastodon_username,
             )
         except MastodonError as mastodon_error:
-            config.bot.logger.error(
-                "Error while logging into Mastodon: %s", mastodon_error
-            )
-            config.bot.logger.error("Tootbot cannot continue, now shutting down")
+            logger.error("Error while logging into Mastodon: %s", mastodon_error)
+            logger.error("Tootbot cannot continue, now shutting down")
             sys.exit(1)
 
     @staticmethod
-    def get_secrets(mastodon_domain: str, logger: logging.Logger) -> None:
+    def get_secrets(mastodon_domain: str) -> None:
         """Checks that Mastodon API secrets are available. If not collects
         neccesarry info from user input the create and store APi secrets for
         Mastodon API secrets.
@@ -115,7 +116,7 @@ class MastodonPublisher:
                 logger.error("Tootbot cannot continue, now shutting down")
                 sys.exit(1)
 
-    def make_post(
+    async def make_post(
         self: MP,
         posts: Dict[Any, Any],
         reddit_helper: RedditHelper,
@@ -138,15 +139,12 @@ class MastodonPublisher:
             for post in source_posts:
 
                 # Find out if we have any attachments to include with toot.
-                self.bot.logger.debug(
+                logger.debug(
                     "Getting attachments for post %s",
                     source_posts[post].id,
                 )
-                attachments = MediaAttachment(
-                    source_posts[post],
-                    media_helper,
-                    self.bot.logger,
-                )
+                attachments = MediaAttachment(source_posts[post], media_helper)
+                await attachments.get_media_files()
                 self._remove_posted_earlier(attachments)
 
                 # Make sure the post contains media,
@@ -203,9 +201,7 @@ class MastodonPublisher:
                     self.mastodon_config.number_of_errors = 0
 
                 except MastodonError as mastodon_error:
-                    self.bot.logger.error(
-                        "Error while posting toot: %s", mastodon_error
-                    )
+                    logger.error("Error while posting toot: %s", mastodon_error)
                     # Log the post anyway, so we don't get into a loop of the
                     # same error
                     self.bot.post_recorder.log_post(reddit_id=source_posts[post].id)
@@ -234,35 +230,33 @@ class MastodonPublisher:
             return media_ids
 
         title = "Uploading attachments"
-        with alive_bar(
-            title=f"{title:.<60}",
+        for checksum, media_path in tqdm(
+            attachments.media_paths.items(),
+            desc=f"{title:.<60}",
             total=len(attachments.media_paths),
-            enrich_print=False,
-            dual_line=True,
-        ) as progress_bar:
-
-            for checksum, media_path in attachments.media_paths.items():
-                progress_bar.text = f"file {media_path} with checksum {checksum}"
-                try:
-                    mime_type = magic.from_file(media_path, mime=True)
-                    self.bot.logger.debug(
-                        "Media %s (%s) with checksum: %s",
-                        media_path,
-                        mime_type,
-                        checksum,
-                    )
-                    media = self.mastodon.media_post(media_path, mime_type)
-                    # Log the media upload
-                    self.bot.post_recorder.log_post(check_sum=checksum)
-                    media_ids.append(media)
-                except (MastodonError, TypeError) as mastodon_error:
-                    self.bot.logger.debug(
-                        "Error when uploading media %s (%s): %s",
-                        media_path,
-                        mime_type,
-                        mastodon_error,
-                    )
-                progress_bar()  # pylint: disable=not-callable
+            unit="attachment",
+            ncols=120,
+            bar_format=PROGRESS_BAR_FORMAT,
+        ):
+            try:
+                mime_type = magic.from_file(media_path, mime=True)
+                logger.debug(
+                    "Media %s (%s) with checksum: %s",
+                    media_path,
+                    mime_type,
+                    checksum,
+                )
+                media = self.mastodon.media_post(media_path, mime_type)
+                # Log the media upload
+                self.bot.post_recorder.log_post(check_sum=checksum)
+                media_ids.append(media)
+            except (MastodonError, TypeError) as mastodon_error:
+                logger.debug(
+                    "Error when uploading media %s (%s): %s",
+                    media_path,
+                    mime_type,
+                    mastodon_error,
+                )
 
         return media_ids
 
@@ -278,7 +272,7 @@ class MastodonPublisher:
         # Build a list of checksums for files that have already been posted earlier
         checksums = []
         for checksum in attachments.media_paths:
-            self.bot.logger.debug(
+            logger.debug(
                 "Media attachment (path, checksum): %s, %s",
                 attachments.media_paths[checksum],
                 checksum,
@@ -287,9 +281,7 @@ class MastodonPublisher:
                 checksums.append(checksum)
             # Check for duplicate of attachment sha256
             elif self.bot.post_recorder.duplicate_check(checksum):
-                self.bot.logger.debug(
-                    "Media with checksum %s has already been posted", checksum
-                )
+                logger.debug("Media with checksum %s has already been posted", checksum)
                 checksums.append(checksum)
         # Remove all empty or previously posted images
         for checksum in checksums:
